#[cfg(feature = "Win32_NetworkManagement_Dhcp")]
#[doc = "Required features: `\"Win32_NetworkManagement_Dhcp\"`"]
pub mod Dhcp;
#[cfg(feature = "Win32_NetworkManagement_Dns")]
#[doc = "Required features: `\"Win32_NetworkManagement_Dns\"`"]
pub mod Dns;
#[cfg(feature = "Win32_NetworkManagement_InternetConnectionWizard")]
#[doc = "Required features: `\"Win32_NetworkManagement_InternetConnectionWizard\"`"]
pub mod InternetConnectionWizard;
#[cfg(feature = "Win32_NetworkManagement_IpHelper")]
#[doc = "Required features: `\"Win32_NetworkManagement_IpHelper\"`"]
pub mod IpHelper;
#[cfg(feature = "Win32_NetworkManagement_Multicast")]
#[doc = "Required features: `\"Win32_NetworkManagement_Multicast\"`"]
pub mod Multicast;
#[cfg(feature = "Win32_NetworkManagement_Ndis")]
#[doc = "Required features: `\"Win32_NetworkManagement_Ndis\"`"]
pub mod Ndis;
#[cfg(feature = "Win32_NetworkManagement_NetBios")]
#[doc = "Required features: `\"Win32_NetworkManagement_NetBios\"`"]
pub mod NetBios;
#[cfg(feature = "Win32_NetworkManagement_NetManagement")]
#[doc = "Required features: `\"Win32_NetworkManagement_NetManagement\"`"]
pub mod NetManagement;
#[cfg(feature = "Win32_NetworkManagement_NetShell")]
#[doc = "Required features: `\"Win32_NetworkManagement_NetShell\"`"]
pub mod NetShell;
#[cfg(feature = "Win32_NetworkManagement_NetworkDiagnosticsFramework")]
#[doc = "Required features: `\"Win32_NetworkManagement_NetworkDiagnosticsFramework\"`"]
pub mod NetworkDiagnosticsFramework;
#[cfg(feature = "Win32_NetworkManagement_P2P")]
#[doc = "Required features: `\"Win32_NetworkManagement_P2P\"`"]
pub mod P2P;
#[cfg(feature = "Win32_NetworkManagement_QoS")]
#[doc = "Required features: `\"Win32_NetworkManagement_QoS\"`"]
pub mod QoS;
#[cfg(feature = "Win32_NetworkManagement_Rras")]
#[doc = "Required features: `\"Win32_NetworkManagement_Rras\"`"]
pub mod Rras;
#[cfg(feature = "Win32_NetworkManagement_Snmp")]
#[doc = "Required features: `\"Win32_NetworkManagement_Snmp\"`"]
pub mod Snmp;
#[cfg(feature = "Win32_NetworkManagement_WNet")]
#[doc = "Required features: `\"Win32_NetworkManagement_WNet\"`"]
pub mod WNet;
#[cfg(feature = "Win32_NetworkManagement_WebDav")]
#[doc = "Required features: `\"Win32_NetworkManagement_WebDav\"`"]
pub mod WebDav;
#[cfg(feature = "Win32_NetworkManagement_WiFi")]
#[doc = "Required features: `\"Win32_NetworkManagement_WiFi\"`"]
pub mod WiFi;
#[cfg(feature = "Win32_NetworkManagement_WindowsConnectionManager")]
#[doc = "Required features: `\"Win32_NetworkManagement_WindowsConnectionManager\"`"]
pub mod WindowsConnectionManager;
#[cfg(feature = "Win32_NetworkManagement_WindowsFilteringPlatform")]
#[doc = "Required features: `\"Win32_NetworkManagement_WindowsFilteringPlatform\"`"]
pub mod WindowsFilteringPlatform;
#[cfg(feature = "Win32_NetworkManagement_WindowsFirewall")]
#[doc = "Required features: `\"Win32_NetworkManagement_WindowsFirewall\"`"]
pub mod WindowsFirewall;
#[cfg(feature = "Win32_NetworkManagement_WindowsNetworkVirtualization")]
#[doc = "Required features: `\"Win32_NetworkManagement_WindowsNetworkVirtualization\"`"]
pub mod WindowsNetworkVirtualization;
