#![allow(unused_imports)]
#![allow(clippy::all)]
use super::*;
use wasm_bindgen::prelude::*;
#[cfg(web_sys_unstable_apis)]
#[wasm_bindgen]
extern "C" {
    #[wasm_bindgen(
        extends = "::js_sys::Object",
        js_name = "Highlight",
        typescript_type = "Highlight"
    )]
    #[derive(Debug, Clone, PartialEq, Eq)]
    #[doc = "The `Highlight` class."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub type Highlight;
    #[cfg(web_sys_unstable_apis)]
    #[wasm_bindgen(method, getter, js_class = "Highlight", js_name = "size")]
    #[doc = "Getter for the `size` field of this object."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/size)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn size(this: &Highlight) -> u32;
    #[cfg(web_sys_unstable_apis)]
    #[wasm_bindgen(method, getter, js_class = "Highlight", js_name = "priority")]
    #[doc = "Getter for the `priority` field of this object."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/priority)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn priority(this: &Highlight) -> i32;
    #[cfg(web_sys_unstable_apis)]
    #[wasm_bindgen(method, setter, js_class = "Highlight", js_name = "priority")]
    #[doc = "Setter for the `priority` field of this object."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/priority)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn set_priority(this: &Highlight, value: i32);
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "HighlightType")]
    #[wasm_bindgen(method, getter, js_class = "Highlight", js_name = "type")]
    #[doc = "Getter for the `type` field of this object."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/type)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `Highlight`, `HighlightType`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn type_(this: &Highlight) -> HighlightType;
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "HighlightType")]
    #[wasm_bindgen(method, setter, js_class = "Highlight", js_name = "type")]
    #[doc = "Setter for the `type` field of this object."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/type)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `Highlight`, `HighlightType`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn set_type(this: &Highlight, value: HighlightType);
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(catch, constructor, variadic, js_class = "Highlight")]
    #[doc = "The `new Highlight(..)` constructor, creating a new instance of `Highlight`."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/Highlight)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn new(initial_ranges: &[AbstractRange]) -> Result<Highlight, JsValue>;
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(method, js_class = "Highlight")]
    #[doc = "The `add()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/add)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn add(this: &Highlight, value: &AbstractRange) -> Highlight;
    #[cfg(web_sys_unstable_apis)]
    #[wasm_bindgen(method, js_class = "Highlight")]
    #[doc = "The `clear()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/clear)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn clear(this: &Highlight);
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(method, js_class = "Highlight")]
    #[doc = "The `delete()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/delete)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn delete(this: &Highlight, value: &AbstractRange) -> bool;
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(catch, method, js_class = "Highlight", js_name = "forEach")]
    #[doc = "The `forEach()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/forEach)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn for_each(
        this: &Highlight,
        callback: &::js_sys::Function<fn(AbstractRange) -> ::js_sys::Undefined>,
    ) -> Result<(), JsValue>;
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(method, js_class = "Highlight")]
    #[doc = "The `has()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/has)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn has(this: &Highlight, value: &AbstractRange) -> bool;
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(method, js_class = "Highlight")]
    #[doc = "The `entries()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/entries)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn entries(
        this: &Highlight,
    ) -> ::js_sys::Iterator<::js_sys::ArrayTuple<(AbstractRange, AbstractRange)>>;
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(method, js_class = "Highlight")]
    #[doc = "The `keys()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/keys)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn keys(this: &Highlight) -> ::js_sys::Iterator<AbstractRange>;
    #[cfg(web_sys_unstable_apis)]
    #[cfg(feature = "AbstractRange")]
    #[wasm_bindgen(method, js_class = "Highlight")]
    #[doc = "The `values()` method."]
    #[doc = ""]
    #[doc = "[MDN Documentation](https://developer.mozilla.org/en-US/docs/Web/API/Highlight/values)"]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `AbstractRange`, `Highlight`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://wasm-bindgen.github.io/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub fn values(this: &Highlight) -> ::js_sys::Iterator<AbstractRange>;
}
